@@ -15,6 +15,10 @@
 #
 #  $ export GST_PLUGIN_PATH=$GST_PLUGIN_PATH:$PWD/plugin:$PWD/examples/plugins
 #  $ GST_DEBUG=python:4 gst-launch-1.0 fakesrc num-buffers=10 ! mysink
+#
+# Since this implements the `mypysink://` protocol you can also do:
+#
+#  $ gst-launch-1.0 videotestsrc ! pymysink://
 
 from gi.repository import Gst, GObject, GstBase
 Gst.init_python()
@@ -22,7 +26,7 @@ Gst.init_python()
 #
 # Simple Sink element created entirely in python
 #
-class MySink(GstBase.BaseSink):
+class MySink(GstBase.BaseSink, Gst.URIHandler):
     __gstmetadata__ = ('CustomSink','Sink', \
                       'Custom test sink element', 'Edward Hervey')
 
@@ -31,9 +35,23 @@ class MySink(GstBase.BaseSink):
                                            Gst.PadPresence.ALWAYS,
                                            Gst.Caps.new_any())
 
+    __protocols__ = ("pymysink",)
+    __uritype__ = Gst.URIType.SINK
+
+    def __init__(self):
+        super().__init__()
+        # Working around https://gitlab.gnome.org/GNOME/pygobject/-/merge_requests/129
+        self.__dontkillme = self
+
     def do_render(self, buffer):
         Gst.info("timestamp(buffer):%s" % (Gst.TIME_ARGS(buffer.pts)))
         return Gst.FlowReturn.OK
 
+    def do_get_uri(self, uri):
+        return "pymysink://"
+
+    def do_set_uri(self, uri):
+        return True
+
 GObject.type_register(MySink)
 __gstelementfactory__ = ("mysink", Gst.Rank.NONE, MySink)
