@@ -25,10 +25,33 @@
 /* include this first, before NO_IMPORT_PYGOBJECT is defined */
 #include <pygobject.h>
 #include <gst/gst.h>
-#include <gmodule.h>
 #include <Python.h>
-
-void *_PyGstElement_Type;
+/*
+ * We need to call dlopen() directly on macOS to workaround a macOS runtime
+ * linker bug. When there are nested dlopen() calls and the second dlopen() is
+ * called from another library (such as gmodule), @loader_path is resolved as
+ * @executable_path and RPATHs are read from the executable (gst-plugin-scanner)
+ * instead of the library itself (libgstges.dylib). This doesn't happen if the
+ * second dlopen() call is directly in the source code of the library.
+ * Previously seen at:
+ * https://gitlab.freedesktop.org/gstreamer/gstreamer/-/issues/1171#note_2290789
+ */
+#ifdef G_OS_WIN32
+#include <gmodule.h>
+#define gstpy_module_open(fname) g_module_open(fname,0)
+#define gstpy_module_error g_module_error
+#define gstpy_module_symbol(module,name,symbol) g_module_symbol(module,name,symbol)
+#else
+#include <dlfcn.h>
+#define gstpy_module_open(fname) dlopen(fname,RTLD_NOW | RTLD_GLOBAL)
+#define gstpy_module_error dlerror
+static inline gboolean
+gstpy_module_symbol (gpointer handle, const char *name, gpointer * symbol)
+{
+  *symbol = dlsym (handle, name);
+  return *symbol != NULL;
+}
+#endif
 
 GST_DEBUG_CATEGORY_STATIC (pyplugindebug);
 #define GST_CAT_DEFAULT pyplugindebug
@@ -212,9 +235,8 @@ static gboolean
 plugin_init (GstPlugin * plugin)
 {
   PyGILState_STATE state = 0;
-  PyObject *gi, *require_version, *args, *gst, *dict, *pyplugin;
+  PyObject *gi, *require_version, *args, *gst, *pyplugin;
   gboolean we_initialized = FALSE;
-  GModule *libpython;
   gpointer has_python = NULL;
   const gchar *override_path;
 
@@ -229,23 +251,19 @@ plugin_init (GstPlugin * plugin)
       GST_PLUGIN_DEPENDENCY_FLAG_NONE);
 
   GST_LOG ("Checking to see if libpython is already loaded");
-  if (g_module_symbol (g_module_open (NULL, G_MODULE_BIND_LOCAL),
+  if (gstpy_module_symbol (gstpy_module_open (NULL),
           "_Py_NoneStruct", &has_python) && has_python) {
     GST_LOG ("libpython is already loaded");
   } else {
-    const gchar *libpython_path =
-        PY_LIB_LOC "/libpython" PYTHON_VERSION PY_ABI_FLAGS "." PY_LIB_SUFFIX;
-    GST_LOG ("loading libpython from '%s'", libpython_path);
-    libpython = g_module_open (libpython_path, 0);
-    if (!libpython) {
-      g_critical ("Couldn't g_module_open libpython. Reason: %s",
-          g_module_error ());
+    GST_LOG ("loading libpython by name: %s", PY_LIB_FNAME);
+    if (!gstpy_module_open (PY_LIB_FNAME)) {
+      GST_ERROR ("Couldn't load libpython. Reason: %s", gstpy_module_error ());
       return FALSE;
     }
   }
 
   if (!Py_IsInitialized ()) {
-    GST_LOG ("python wasn't initialized");
+    GST_LOG ("python wasn't already initialized");
     /* set the correct plugin for registering stuff */
     Py_Initialize ();
     we_initialized = TRUE;
@@ -274,7 +292,20 @@ plugin_init (GstPlugin * plugin)
 
   GST_LOG ("initializing pygobject");
   if (!pygobject_init (3, 0, 0)) {
-    g_critical ("pygobject initialization failed");
+    if (PyErr_Occurred ()) {
+      PyObject *type, *value, *traceback;
+      PyErr_Fetch (&type, &value, &traceback);
+      PyObject *str = PyObject_Str (value);
+      if (str) {
+        g_critical ("pygobject initialization failed: %s\n",
+            PyUnicode_AsUTF8 (str));
+        Py_DECREF (str);
+      }
+      Py_XDECREF (type);
+      Py_XDECREF (value);
+      Py_XDECREF (traceback);
+      PyErr_Clear ();
+    }
     return FALSE;
   }
 
@@ -300,25 +331,6 @@ plugin_init (GstPlugin * plugin)
   }
 
   if (we_initialized) {
-    PyObject *tmp;
-
-    dict = PyModule_GetDict (gst);
-    if (!dict) {
-      g_critical ("gi.repository.Gst is no dict");
-      return FALSE;
-    }
-
-    tmp =
-        PyObject_GetAttr (PyMapping_GetItemString (dict,
-            "_introspection_module"), PyUnicode_FromString ("__dict__"));
-
-    _PyGstElement_Type = PyMapping_GetItemString (tmp, "Element");
-
-    if (!_PyGstElement_Type) {
-      g_critical ("Could not get Gst.Element");
-      return FALSE;
-    }
-
     pyplugin = pygobject_new (G_OBJECT (plugin));
     if (!pyplugin || PyModule_AddObject (gst, "__plugin__", pyplugin) != 0) {
       g_critical ("Couldn't set __plugin__ attribute");
